@@ -2,10 +2,11 @@
 
 ###############################################################
 #  PyNLPl - Text Processors
-#       by Maarten van Gompel (proycon)
-#       http://ilk.uvt.nl/~mvgompel
-#       Induction for Linguistic Knowledge Research Group
-#       Universiteit van Tilburg
+#   by Maarten van Gompel
+#   Centre for Language Studies
+#   Radboud University Nijmegen
+#   http://www.github.com/proycon/pynlpl
+#   proycon AT anaproy DOT nl
 #       
 #       Licensed under GPLv3
 # 
@@ -13,50 +14,40 @@
 #
 ###############################################################
 
+
+from __future__ import print_function
+from __future__ import unicode_literals
+from __future__ import division
+from __future__ import absolute_import
+#from pynlpl.common import u
+import sys
+if sys.version < '3':
+    from codecs import getwriter
+    stderr = getwriter('utf-8')(sys.stderr)
+    stdout = getwriter('utf-8')(sys.stdout)
+else:
+    stderr = sys.stderr
+    stdout = sys.stdout
+
 import unicodedata
 import string
-import codecs
+import io
 import array
-from statistics import FrequencyList
-from datatypes import intarraytobytearray, bytearraytoint, containsnullbyte
+from itertools import permutations
+from pynlpl.statistics import FrequencyList
+from pynlpl.datatypes import intarraytobytearray, bytearraytoint, containsnullbyte
 
-try:
-    from itertools import permutations
-except ImportError: #python too old, include function explicitly (code from from Python Documentation)
-    def permutations(iterable, r=None): 
-        # permutations('ABCD', 2) --> AB AC AD BA BC BD CA CB CD DA DB DC
-        # permutations(range(3)) --> 012 021 102 120 201 210
-        pool = tuple(iterable)
-        n = len(pool)
-        r = n if r is None else r
-        if r > n:
-            return
-        indices = range(n)
-        cycles = range(n, n-r, -1)
-        yield tuple(pool[i] for i in indices[:r])
-        while n:
-            for i in reversed(range(r)):
-                cycles[i] -= 1
-                if cycles[i] == 0:
-                    indices[i:] = indices[i+1:] + indices[i:i+1]
-                    cycles[i] = n - i
-                else:
-                    j = cycles[i]
-                    indices[i], indices[-j] = indices[-j], indices[i]
-                    yield tuple(pool[i] for i in indices[:r])
-                    break
-            else:
-                return
             
             
-class Windower:
+class Windower(object):
     """Moves a sliding window over a list of tokens, returning all ngrams"""
 
     def __init__(self, tokens, n=1, beginmarker = "<begin>", endmarker = "<end>"):
-        if isinstance(tokens, str) or  isinstance(unicode, str):
+        if isinstance(tokens, str) or (sys.version < '3' and isinstance(tokens, unicode)):
             self.tokens = tuple(tokens.split())
         else:
             self.tokens = tuple(tokens)
+        assert isinstance(n, int)
         self.n = n
         self.beginmarker = beginmarker
         self.endmarker = endmarker
@@ -69,7 +60,7 @@ class Windower:
         if self.endmarker:
             endmarker = (self.endmarker),  #tuple
 
-        for i in xrange(-(self.n - 1),l):
+        for i in range(-(self.n - 1),l):
             begin = i
             end = i + self.n
             if begin >= 0 and end <= l:
@@ -94,10 +85,12 @@ class MultiWindower(object):
     "Extract n-grams of various configurations from a sequence"
     
     def __init__(self,tokens, min_n = 1, max_n = 9, beginmarker=None, endmarker=None):
-        if isinstance(tokens, str) or  isinstance(unicode, str):
+        if isinstance(tokens, str) or (sys.version < '3' and isinstance(tokens, unicode)):
             self.tokens = tuple(tokens.split())
         else:
             self.tokens = tuple(tokens)
+        assert isinstance(min_n, int)
+        assert isinstance(max_n, int)
         self.min_n = min_n
         self.max_n = max_n
         self.beginmarker = beginmarker
@@ -110,15 +103,15 @@ class MultiWindower(object):
 
 
 class ReflowText(object):
+    """Attempts to re-flow a text that has arbitrary line endings in it. Also undoes hyphenisation"""
+    
     def __init__(self, stream, filternontext=True):
         self.stream = stream
         self.filternontext = filternontext
         
     def __iter__(self):
-        eosmarkers = ('.',':','?','!','"',"'",u"„",u"”",u"’")
+        eosmarkers = ('.',':','?','!','"',"'","„","”","’")
         emptyline = 0
-        pagebreak = False
-        niceeos = True
         buffer = ""
         for line in self.stream:
             
@@ -198,10 +191,15 @@ def calculate_overlap(haystack, needle, allowpartial=True):
 
     return solutions
             
-    
 
-def crude_tokenizer(line):
-    """This is a very crude tokenizer"""
+def crude_tokenizer(line):    
+    return tokenise(line) #backwards-compatibility
+
+def tokenize(line):
+    return tokenise(line)
+
+def tokenise(line):
+    """A simple tokeniser"""
     tokens = []
     buffer = ''
     for c in line.strip():
@@ -209,20 +207,25 @@ def crude_tokenizer(line):
             if buffer:
                 tokens.append(buffer)
                 buffer = ''
+            if c in string.punctuation:
+                tokens.append(c)
         else:
             buffer += c          
-    if buffer: tokens.append(buffer)  
+    if buffer: 
+        tokens.append(buffer)  
     return tokens
     
 
 def strip_accents(s, encoding= 'utf-8'):
     """Strip characters with diacritics and return a flat ascii representation"""
-    if isinstance(s,unicode):
-       return unicodedata.normalize('NFKD', s).encode('ASCII', 'ignore')
+    if sys.version < '3':
+        if isinstance(s,unicode):
+           return unicodedata.normalize('NFKD', s).encode('ASCII', 'ignore')
+        else:
+           return unicodedata.normalize('NFKD', unicode(s,encoding)).encode('ASCII', 'ignore')
     else:
-       return unicodedata.normalize('NFKD', unicode(s,encoding)).encode('ASCII', 'ignore')
-
-
+        if isinstance(s,bytes): s = str(s,encoding)
+        return str(unicodedata.normalize('NFKD', s).encode('ASCII', 'ignore'),'ascii')
 
 def swap(tokens, maxdist=2):
     """Perform a swap operation on a sequence of tokens, exhaustively swapping all tokens up to the maximum specified distance. This is a subset of all permutations."""
@@ -286,20 +289,20 @@ class Classer(object):
             self.newestclass = 1 #0 and 1 are reserved for space and newline
                     
         if isinstance(f, FrequencyList):                        
-            for word, _ in f:  
+            for word, _ in f:
                 self.newestclass += 1
                 if self.filesupport:
                     while containsnullbyte(self.newestclass): 
                         self.newestclass += 1
-                print self.newestclass, word
+                print(self.newestclass, word,file=stderr)
                 if self.decoder:
                     self.class2word[self.newestclass] = word  
                 if self.encoder:
                     self.word2class[word] = self.newestclass
             if not self.decoder:
                 del self.class2word
-        elif isinstance(f, str):
-            f = codecs.open(f,'r','utf-8')      
+        elif isinstance(f, str) or (sys.version < '3' and isinstance(f,unicode)):
+            f = io.open(f,'r',encoding='utf-8')      
             for line in f:
                 cls, word = line.strip().split('\t',2)
                 if self.decoder: self.class2word[int(cls)] = word
@@ -314,9 +317,9 @@ class Classer(object):
         """Save to a class file"""
         if not self.decoder: raise Exception("Decoder not enabled!")
         if self.encoding:
-            f = codecs.open(filename,'w',self.encoding)   
+            f = io.open(filename,'w',encoding=self.encoding)   
         else:
-            f = open(filename,'w')
+            f = io.open(filename,'w',encoding='utf-8')
         for cls, word in sorted(self.class2word.items()):
             if cls:
                 f.write( str(cls) + '\t' + word + '\n')
@@ -359,8 +362,8 @@ class Classer(object):
     def encodefile(self, fromfile, tofile):
         """Encode a file, converting word tokens to class integers"""
         assert self.filesupport
-        ffrom = open(fromfile,'r')  
-        fto = open(tofile,'w')
+        ffrom = io.open(fromfile,'rb')  
+        fto = io.open(tofile,'wb')
         for line in ffrom:
             a = intarraytobytearray( self.encodeseq( line.strip().split(' ') ))
             a.append(0) #delimiter
@@ -372,7 +375,7 @@ class Classer(object):
         
     def decodefile(self, filename):
         """Decode a file, converting class integers to token strings"""
-        f = open(filename)
+        f = io.open(filename,'rb')
         buffer = array.array('B')
         line = []
         b = chr(0)
